@@ -25,8 +25,11 @@ latrend = function(method,
                     ...,
                     envir = NULL,
                     verbose = getOption('latrend.verbose')) {
+  assert_that(
+    is_class_defined(method),
+    is.lcMethod(method)
+  )
   envir = lcMethod.env(method, parent.frame(), envir)
-  assert_that(is.lcMethod(method))
 
   verbose = as.Verbose(verbose)
   argList = list(...)
@@ -88,6 +91,7 @@ latrend = function(method,
 
 fitLatrendMethod = function(method, data, envir, mc, verbose) {
   assert_that(
+    is_class_defined(method),
     is.lcMethod(method),
     is.data.frame(data),
     is.call(mc),
@@ -115,6 +119,8 @@ fitLatrendMethod = function(method, data, envir, mc, verbose) {
     verbose = verbose
   )
 
+  assert_that(is_class_defined(model))
+
   model@call = do.call(call,
                        c(
                          'latrend',
@@ -142,10 +148,9 @@ fitLatrendMethod = function(method, data, envir, mc, verbose) {
 #' @description Performs a repeated fit of the specified latrend model on the given data.
 #' @inheritParams latrend
 #' @param .rep The number of repeated fits.
-#' @param .errorhandling How to handle fits in which on error occurs.
-#' If `"remove"`, errors are ignored and the respective repetition is exempt from the returned model list.
-#' If `"stop"`, errors are not caught, ensuring that the function halts on the first error.
+#' @param .errorHandling Whether to `"stop"` on an error, or to `"remove'` evaluations that raised an error.
 #' @param .seed Set the seed for generating the respective seed for each of the repeated fits.
+#' @param .parallel Whether to use parallel evaluation. See \link{latrend-parallel}.
 #' @details This method is faster than repeatedly calling [latrend] as it only prepares the data via `prepareData()` once.
 #' @return A `lcModels` object containing the resulting models.
 #' @examples
@@ -159,16 +164,19 @@ latrendRep = function(method,
                        data,
                        .rep = 10,
                        ...,
-                       .errorhandling = 'remove',
+                       .errorHandling = 'stop',
                        .seed = NULL,
+                       .parallel = FALSE,
                        envir = NULL,
                        verbose = getOption('latrend.verbose')) {
   envir = lcMethod.env(method, parent.frame(), envir)
-  assert_that(is.lcMethod(method),
-              is.count(.rep))
+  assert_that(
+    is.lcMethod(method),
+    is.count(.rep),
+    is.flag(.parallel)
+  )
 
   verbose = as.Verbose(verbose)
-  errh = match.arg(.errorhandling, c('stop', 'remove'))
   argList = list(...)
   argList$envir = envir
   newmethod = do.call(update, c(object = method, argList))
@@ -192,13 +200,11 @@ latrendRep = function(method,
 
   # seed
   if (hasName(cmethod, 'seed')) {
-    warning(
-      'The supplied lcMethod object defines a seed, which will result in repeated identical results. Use the .seed argument of latrendRep() to generate different seeds for the repetitions in a reproducible way.'
-    )
+    warning('The supplied lcMethod object defines a seed, which will result in repeated identical results.
+      Use the .seed argument of latrendRep() to generate different seeds for the repetitions in a reproducible way.')
   }
 
-  cat(verbose,
-      sprintf('Generating method seeds for seed = %s.', as.character(.seed)))
+  cat(verbose, sprintf('Generating method seeds for seed = %s.', as.character(.seed)))
   localRNG(seed = .seed, {
     repSeeds = sample.int(.Machine$integer.max,
                           size = .rep,
@@ -208,6 +214,7 @@ latrendRep = function(method,
   id = idVariable(cmethod)
   time = timeVariable(cmethod)
   response = responseVariable(cmethod)
+
   assert_that(
     is.character(idVariable(cmethod)),
     is.character(timeVariable(cmethod)),
@@ -231,17 +238,20 @@ latrendRep = function(method,
                         verbose = verbose)
   exit(verbose)
 
+  `%infix%` = ifelse(.parallel, `%dopar%`, `%do%`)
+
   models = foreach(
     i = seq_len(.rep),
     iseed = repSeeds,
     .combine = c,
-    .errorhandling = errh
-  ) %do% {
+    .errorhandling = .errorHandling
+  ) %infix% {
     cat(verbose,
         sprintf('Fitting model %d/%d for seed %s...',
         i,
         .rep,
         as.character(iseed)))
+    assert_that(is_class_defined(cmethod))
     imethod = update(cmethod, seed = iseed, .eval = TRUE)
     model = fitLatrendMethod(
       imethod,
@@ -266,6 +276,8 @@ latrendRep = function(method,
 #' @param methods A `list` of `lcMethod` objects.
 #' @param data A `data.frame`, `matrix`, or a `list` thereof to which to apply to the respective `lcMethod`. Multiple datasets can be supplied by encapsulating the datasets using `data=.(df1, df2, ..., dfN)`.
 #' @param cartesian Whether to fit the provided methods on each of the datasets. If `cartesian=FALSE`, only a single dataset may be provided or a list of data matching the length of `methods`.
+#' @param parallel Whether to enable parallel evaluation. See \link{latrend-parallel}.
+#' @param errorHandling Whether to `"stop"` on an error, or to `"remove'` evaluations that raised an error.
 #' @param envir The `environment` in which to evaluate the `lcMethod` arguments.
 #' @return A `lcModels` object.
 #' @examples
@@ -282,6 +294,8 @@ latrendRep = function(method,
 latrendBatch = function(methods,
                          data,
                          cartesian = TRUE,
+                         parallel = FALSE,
+                         errorHandling = 'stop',
                          envir = NULL,
                          verbose = getOption('latrend.verbose')) {
   if (!is.list(methods)) {
@@ -293,8 +307,8 @@ latrendBatch = function(methods,
   )), msg = 'methods argument must be a list of lcMethod objects')
   assert_that(
     !missing(data),
-    is.logical(cartesian),
-    is.scalar(cartesian)
+    is.flag(cartesian),
+    is.flag(parallel)
   )
 
   envir = lcMethod.env(methods[[1]], parent.frame(), envir)
@@ -303,8 +317,7 @@ latrendBatch = function(methods,
   nModels = length(methods)
   mc = match.call()[-1]
 
-  header(verbose,
-         sprintf('Batch estimation (N=%d) for longitudinal clustering', nModels))
+  header(verbose, sprintf('Batch estimation (N=%d) for longitudinal clustering', nModels))
 
   dataCall = mc$data
   if (is.name(dataCall)) {
@@ -331,25 +344,39 @@ latrendBatch = function(methods,
   cat(verbose, 'Calling latrend for each method...')
   pushState(verbose)
 
-  models = vector('list', nModels * ifelse(cartesian, nData, 1))
+  # generate method and data lists
+  if (cartesian) {
+    allMethods = methods[rep(seq_len(nModels), nData)]
+    allDataOpts = dataList[rep(seq_len(nData), nModels)]
+  } else if (nModels == nData) {
+    allMethods = methods
+    allDataOpts = dataList
+  } else {
+    # replicate methods and data such that they are equal length
+    allMethods = methods[rep_len(seq_len(nModels), length.out = max(nModels, nData))]
+    allDataOpts = dataList[rep_len(seq_len(nData), length.out = max(nModels, nData))]
+  }
+  assert_that(length(allMethods) == length(allDataOpts))
 
-  for (m in seq_along(methods)) {
-    if (cartesian) {
-      dOpts = seq_along(dataList)
-    } else {
-      dOpts = min(m, nData, nModels)
-    }
-    for (d in dOpts) {
-      cl = do.call(call,
-                   c(
-                     'latrend',
-                     method = quote(methods[[m]]),
-                     data = quote(dataList[[d]]),
-                     envir = quote(envir),
-                     verbose = quote(verbose)
-                   ))
-      models[[(m - 1) * length(dOpts) + ifelse(cartesian, d, 1)]] = eval(cl, envir = parent.frame())
-    }
+  # generate calls
+  allCalls = vector('list', length(allMethods))
+  for (i in seq_along(allMethods)) {
+    allCalls[[i]] = do.call(call,
+      c(
+        'latrend',
+        method = allMethods[[i]],
+        data = quote(allDataOpts[[i]]),
+        envir = quote(envir),
+        verbose = quote(verbose)
+      ))
+  }
+
+  `%infix%` = ifelse(parallel, `%dopar%`, `%do%`)
+  penv = parent.frame()
+
+  models = foreach(cl = allCalls, .packages = 'latrend', .errorhandling = errorHandling, .export = 'envir') %infix% {
+    model = eval(cl, envir = penv)
+    model
   }
 
   popState(verbose)
@@ -357,7 +384,6 @@ latrendBatch = function(methods,
   as.lcModels(models)
 }
 
-
 #' @export
 #' @title Cluster longitudinal data using bootstrapping
 #' @description Performs bootstrapping, generating samples from the given data at the id level, fitting a lcModel to each sample.
@@ -365,6 +391,7 @@ latrendBatch = function(methods,
 #' @param data A `data.frame`.
 #' @param samples The number of bootstrap samples to evaluate.
 #' @param seed The seed to use. Optional.
+#' @inheritParams latrendBatch
 #' @return A `lcModels` object of length `samples`.
 #' @examples
 #' data(latrendData)
@@ -376,6 +403,8 @@ latrendBoot = function(method,
                         data,
                         samples = 50,
                         seed = NULL,
+                        parallel = FALSE,
+                        errorHandling = 'stop',
                         envir = NULL,
                         verbose = getOption('latrend.verbose')) {
   assert_that(is.lcMethod(method), msg = 'method must be lcMethod object (e.g., lcMethodKML("Y") )')
@@ -426,6 +455,8 @@ latrendBoot = function(method,
       methods = methods,
       data = enquote(dataCall),
       cartesian = FALSE,
+      parallel = parallel,
+      errorHandling = errorHandling,
       envir = quote(envir),
       verbose = verbose
     )
@@ -458,6 +489,8 @@ latrendCV = function(method,
                       data,
                       folds = 10,
                       seed = NULL,
+                      parallel = FALSE,
+                      errorHandling = 'stop',
                       envir = NULL,
                       verbose = getOption('latrend.verbose')) {
   assert_that(!missing(data), msg = 'data must be specified')
@@ -495,6 +528,8 @@ latrendCV = function(method,
                    list(
                      method = method,
                      data = dataCall,
+                     parallel = parallel,
+                     errorHandling = errorHandling,
                      verbose = verbose
                    ), envir = parent.frame())
 
