@@ -4,7 +4,7 @@
 #' @name interface-akmedoids
 #' @rdname interface-akmedoids
 #' @title akmedoids interface
-#' @seealso [lcMethodAkmedoids] \link[akmedoids]{akmedoids.clust}
+#' @seealso [lcMethodAkmedoids] \link[akmedoids]{akclustr}
 #' @keywords internal
 NULL
 
@@ -16,7 +16,8 @@ setClass('lcMethodAkmedoids', contains = 'lcMatrixMethod')
 #' @inheritParams lcMethodCustom
 #' @inheritParams lcMethodKML
 #' @param clusterCenter A function for computing the cluster center representation.
-#' @param ... Arguments passed to [akmedoids::akmedoids.clust].
+#' @param crit Criterion to apply for internal model selection. Not applicable.
+#' @param ... Arguments passed to [akmedoids::akclustr].
 #' The following external arguments are ignored: traj, id_field, k
 #' @examples
 #' library(akmedoids)
@@ -31,11 +32,12 @@ lcMethodAkmedoids = function(response,
                              id = getOption('latrend.id'),
                              nClusters = 3, # must be > 2
                              clusterCenter = median,
+                             crit = 'Calinski_Harabasz', # Default silhouette width results in: Error in smooth.spline(x, y) : 'tol' must be strictly positive and finite
                              ...) {
   lcMethod.call(
     'lcMethodAkmedoids',
     call = match.call.defaults(),
-    defaults = akmedoids::akmedoids.clust,
+    defaults = akmedoids::akclustr,
     excludeArgs = c('traj', 'id_field', 'k')
   )
 }
@@ -49,7 +51,7 @@ setMethod('getShortName', signature('lcMethodAkmedoids'), function(object) 'akm'
 #' @rdname interface-akmedoids
 #' @inheritParams fit
 setMethod('fit', signature('lcMethodAkmedoids'), function(method, data, envir, verbose, ...) {
-  args = as.list(method, args = akmedoids::akmedoids.clust)
+  args = as.list(method, args = akmedoids::akclustr)
   args$traj = envir$dataMat
   args$k = method$nClusters
   args$id_field = FALSE
@@ -58,17 +60,19 @@ setMethod('fit', signature('lcMethodAkmedoids'), function(method, data, envir, v
   suppressFun = ifelse(as.logical(verbose), force, capture.output)
 
   suppressFun({
-    model = do.call(akmedoids::akmedoids.clust, args)
+    model = do.call(akmedoids::akclustr, args)
   })
 
   clusNames = make.clusterNames(method$nClusters)
 
-  assert_that(!is.null(model$membership), msg = 'no membership output returned')
+  trajAssignments = model$solutions[[1]]
+  assert_that(length(trajAssignments) > 0, msg = 'no membership output returned')
 
   lcModelCustom(
     data,
-    trajectoryAssignments = factor(model$membership, levels = LETTERS[1:method$nClusters], labels =
-                                  clusNames),
+    trajectoryAssignments = factor(trajAssignments,
+      levels = LETTERS[1:method$nClusters],
+      labels = clusNames),
     clusterTrajectories = method$clusterCenter,
     response = responseVariable(method),
     time = timeVariable(method),
